import Foundation

public struct DecodeStep {
    
    weak var base64Service: Base64Service?
    
    public init(base64Service: Base64Service) {
        self.base64Service = base64Service
    }
    
    public func make(queue: DispatchQueue? = nil, next: Step? = nil) -> Step {
        return { io in
            
            guard let encoded = io["message"] as? String else {
                preconditionFailure()
            }
            
            self.base64Service?.decode(queue: queue, encoded: encoded) { result in
                
                guard case .success(let message) = result else {
                    return
                }
                
                next?(["message": message])
            }
        }
    }
    
    public func successful(queue: DispatchQueue? = nil) -> SuccessfulStep {
        return { next in
            return self.make(next: next)
        }
    }
}
